'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.ACTION_TYPES = undefined;

var _smartWebsocket = require('./smartWebsocket');

var _smartWebsocket2 = _interopRequireDefault(_smartWebsocket);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ACTION_TYPES = exports.ACTION_TYPES = {
	ON_OPEN: '@@SOCKET.ON_OPEN',
	ON_CLOSE: '@@SOCKET.ON_CLOSE'
};

// if host is localhost connect directly to the localhost backend
// else connect to the actual host
var host = window.location.host;

var protocol = 'ws';
if (window.location.protocol === 'https:') {
	protocol = 'wss';
}

/**
 * select part of the state to create patch between two state
 *
 * @param socketPath {string} path to websocket
 * @param actionListeners {array<function>} function exectuted
 * on each action going trought this middleware
 * @param socketListener {array<function>} function executed
 * on each message send to this ws listenner
 *
 * @return {object} result
 *
 */
function createWebsocketMiddleware(socketPath) {
	var actionListeners = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
	var socketListener = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

	var buffer = [];
	var ws = void 0;
	var urlPrefix = protocol + '://' + host + socketPath;

	function send() {
		actionListeners.forEach(function (actionListener) {
			buffer.forEach(function (entrie, key) {
				var previousState = entrie.previousState,
				    action = entrie.action;

				var nextState = entrie.nextState;
				var nextEntry = buffer[key + 1];
				if (nextEntry) {
					nextState = nextEntry.previousState;
				}
				var message = actionListener(previousState, action, nextState);
				if (message) {
					ws.send(message);
				}
			});
		});
		buffer.length = 0;
	}

	return function (_ref) {
		var getState = _ref.getState,
		    dispatch = _ref.dispatch;
		return function (next) {
			return function (action) {
				if (!ws) {
					ws = new _smartWebsocket2.default(urlPrefix, {
						onOpen: function onOpen() {
							return dispatch({ type: ACTION_TYPES.ON_OPEN });
						},
						onClose: function onClose() {
							return dispatch({ type: ACTION_TYPES.ON_CLOSE });
						},
						onMessage: function onMessage(messageEvent) {
							socketListener.forEach(function (func) {
								return func(messageEvent, dispatch, getState);
							});
						}
					});
				}
				var entrie = {};
				buffer.push(entrie);
				entrie.action = action;
				entrie.previousState = getState();
				var result = next(action);
				entrie.nextState = getState();
				send();
				return result;
			};
		};
	};
}

exports.default = createWebsocketMiddleware;